#pragma once

#include <Core/World.hpp>
#include <Core/Chunk.hpp>
#include <JGL/JGL.h>
#include <Client/Camera2D.hpp>
#include <Core/SimpleAABBSolver.hpp>
#include <thread>
#include <Core/ConcurrentQueue.hpp>
#include <Core/ThreadPool.hpp>

namespace CaveGame::Client
{
    using Core::ConcurrentQueue;
    using namespace std::chrono_literals;




    class AsyncChunkRenderer : public ThreadPool
    {
    public:

    };

    class LocalWorld : public CaveGame::Core::World
    {
    public:
        //JGL::Font font;
        LocalWorld(const std::string& world_name, int seed, bool overwrite);
        void PostInit();
        void Draw();
        void Update(float elapsed) override;
        void RefreshAll() override;
        Camera2D camera;
        std::thread chunk_thread;
        std::atomic<bool> run_chunk_thread = true;
        ConcurrentQueue<Vector2> RequestedChunks;
        std::vector<Vector2> chunks_in_waiting;
        ConcurrentQueue<Core::Chunk> ServedChunks;
        Vector2 mouse_pos;

    protected:

        std::unordered_map<Vector2, JGL::RenderTarget*> cached_chunk_sprites;

        bool IsChunkCellWithinViewport(const Vector2& coords) const;

        // TODO: Chunk should render itself, no? Logical Separation issue here aswell...
        void RenderChunk(const Vector2& coords, const Core::Chunk& chunk);

        void LookForChunksNeedLoading();
        void LookForChunksNeedUnloading();

        void RequestChunk(const Vector2& cell);


        void ChunkServerThread();

        void RenderChunkTexture(const Vector2 &coords, JGL::RenderTarget* destination, const Core::Chunk& chunk);

        /// Render chunk boundaries and relative coordinates, around the camera.
        void DrawChunkGrid() const;
        /// Checks for missing, or out-of-date, cached sprites of chunks, and renders them.
        void CheckCachedChunkSprites();

        bool AwaitingChunkAtCell(const Vector2 &cell);
    };
}