#include <Client/GameSession.hpp>
#include <Core/Loggers.hpp>
#include <JUI/Widgets/Rect.hpp>
#include <JUI/Widgets/ListLayout.hpp>
#include <JUI/UDim.hpp>
#include <JUI/UDim2.hpp>
#include "JUI/Widgets/TextRect.hpp"
#include "JUI/UDim.hpp"

void CaveGame::Client::GameSession::Unload() {
    Scene::Unload();
    UnloadHUD();
}

void CaveGame::Client::GameSession::Load() {
    Scene::Load();
    ConstructHUD();
    world->PostInit();
}

void CaveGame::Client::GameSession::Draw() {
    world->Draw();
    hud->Draw();
    //for (auto& entity: entity_list)
    //{
    //    entity->Draw();
    //}
}

void CaveGame::Client::GameSession::SaveAndExit()
{
    world->SaveAndExit();
}

void CaveGame::Client::GameSession::Update(float elapsed) {
    world->Update(elapsed);
    hud->Update(elapsed);
    HUDTick();
    //for (auto& entity: entity_list)
    //{
    //    entity->Update(elapsed);
    //}
}



CaveGame::Client::GameSession::GameSession(bool overwite_world) : Scene() {
    world = new LocalWorld("test_world", 0, overwite_world);
    //hud = new JUI::Scene();
}

void CaveGame::Client::GameSession::PassWindowSize(const Vector2 &size) {
    world->camera.PassViewportSize(size);
    hud->SetViewportSize(size);
}

void CaveGame::Client::GameSession::PassKeyInput(Key key, bool pressed) {
    // DO NOT chain input conditions with else; you will introduce biasing

    if (key == Keys::One)
        active_hotbar_slot = 0;
    if (key == Keys::Two)
        active_hotbar_slot = 1;
    if (key == Keys::Three)
        active_hotbar_slot = 2;
    if (key == Keys::Four)
        active_hotbar_slot = 3;
    if (key == Keys::Five)
        active_hotbar_slot = 4;
    if (key == Keys::Six)
        active_hotbar_slot = 5;
    if (key == Keys::Seven)
        active_hotbar_slot = 6;
    if (key == Keys::Eight)
        active_hotbar_slot = 7;
    if (key == Keys::Nine)
        active_hotbar_slot = 8;
    if (key == Keys::Zero)
        active_hotbar_slot = 9;


    /*if (key == Keys::LeftArrow)
    {
        world->camera.MoveLeft();
    }
    if (key == Keys::RightArrow)
    {
        world->camera.MoveRight();
    }
    if (key == Keys::UpArrow)
    {
        world->camera.MoveUp();
    }
    if (key == Keys::DownArrow)
    {
        world->camera.MoveDown();
    }*/
}

void CaveGame::Client::GameSession::ConstructHUD() {

    Logs::Info("Building player HUD.");

    hud = new JUI::Scene();

    auto* hotbar_rect = new JUI::Rect(hud);
    hotbar_rect->BGColor(Colors::LightGray);
    hotbar_rect->BorderColor(Colors::White);
    hotbar_rect->SetBorderWidth(1.5f);
    hotbar_rect->CornerRounding(8);
    hotbar_rect->Size({488_px, 18_px});
    hotbar_rect->Position({50_percent, 98_percent});
    hotbar_rect->AnchorPoint({0.5f, 1.f});
    hotbar_rect->BorderMode(JUI::BorderMode::Outline);

    item_label = new JUI::TextRect(hotbar_rect);
    item_label->BGColor({0,0,0,0});
    item_label->AnchorPoint({1,0});
    item_label->SetContent("Item Name Here");
    item_label->SetTextColor(Colors::White);
    item_label->SetTextSize(22);
    item_label->Size({200_px, 30_px});
    item_label->Position({0_percent, JUI::UDim(-25, 0)});
    item_label->AlignRight();

    auto* layout = new JUI::HorizontalListLayout(hotbar_rect);
    layout->PaddingLeft(4_px);
    layout->PaddingRight(4_px);

    for (int i = 0; i < 10; i++)
    {
        auto* cell = new JUI::Rect(layout);
        cell->Name("hotbar_" + std::to_string(i));
        cell->BGColor(Colors::LightGray);
        cell->BorderColor(Colors::White);
        cell->Size({48_px, 48_px});
        cell->AnchorPoint({0.f, 0.8f});
        cell->CornerRounding(5);
        cell->SetBorderWidth(1.5f);
        cell->BorderMode(JUI::BorderMode::Outline);
        cell->PaddingBottom(10_px);
        cell->PaddingRight(5_px);

        auto* amount_label = new JUI::TextRect(cell);
        amount_label->BGColor({0,0,0,0});
        amount_label->AnchorPoint({1,1});
        amount_label->SetContent("99x");
        amount_label->SetTextColor(Colors::White);
        amount_label->SetTextSize(18);
        amount_label->Size({30_px, 30_px});
        amount_label->Position({95_percent, 95_percent});
        amount_label->AlignRight();

        hotbar_elements[i] = cell;
    }

}

void CaveGame::Client::GameSession::UnloadHUD() {
    Logs::Info("Unloading player HUD.");
    delete hud;
}
