#include <Client/MainMenu.hpp>
#include <JUI/Widgets/TextRect.hpp>
#include "JUI/Widgets/ListLayout.hpp"
#include "JUI/Widgets/Button.hpp"

CaveGame::Client::MainMenu::MainMenu() : Scene()
{
    using namespace JUI;

    scene = new JUI::Scene();
    scene->Visible(true);
}

void CaveGame::Client::MainMenu::Update(float elapsed) {

    scene->Update(elapsed);

    goal = Vector2::Clamp(Vector2::Zero, goal, {window_size.x, window_size.y});
    mpos = Vector2::Lerp(mpos, goal, elapsed);
}

void CaveGame::Client::MainMenu::Draw() {

    JGL::J2D::Begin();

    auto aspect = window_size.x / bg->GetDimensions().x*2;

    glClearColor(0.f,0.f,0.f,1.f);
    JGL::J2D::DrawSprite(*bg, {-(mpos.x/2.f),-mpos.y*2.f}, 0.f, {0,0}, {aspect,aspect});
    JGL::J2D::End();
    scene->Draw();

}

void CaveGame::Client::MainMenu::Load() {

    bg = new JGL::Texture("assets/textures/bg.png");

    Scene::Load();
}

void CaveGame::Client::MainMenu::Unload() {
    Scene::Unload();
}

template <typename T>
T* create_widget(JUI::Widget* parent)
{
    return new T(parent);
}

JUI::TextButton* CaveGame::Client::MainMenu::create_mainmenu_btn(const std::string& content, JUI::Widget* parent)
{
    auto* btn = create_widget<JUI::TextButton>(parent);
    btn->SetFont(JGL::Fonts::Jupiteroid);
    btn->SetTextSize(24);
    btn->SetTextColor(Colors::White);
    btn->BaseBGColor(Colors::LightGray);
    btn->HoveredBGColor(Colors::Blues::LightSteelBlue);
    btn->BGColor(btn->BaseBGColor());

    btn->Size({0, 40, 1.f, 0});
    btn->Enable();
    btn->Center();
    btn->SetContent(content);
    return btn;
}

void CaveGame::Client::MainMenu::BuildWidgets() {

    using namespace JUI;

    title = new TextRect(scene);
    title->SetFont(JGL::Fonts::Jupiteroid);
    title->SetContent("CaveGame");
    title->SetTextColor(Colors::White);
    title->SetTextSize(64);
    title->Position({0, 0, 0.5f, 0.2f});
    title->Size({0, 0, 0.3f, 0.15f});
    title->AnchorPoint({0.5f, 0.5f});
    title->Center();
    title->BGColor({0,0,0,0});
    title->SetBorderStyling(Colors::Cyans::Cyan, 1);

    button_group = new Rect(scene);
    button_group->Size({250, 400, 0, 0});
    button_group->Position({0, 0, 0.5f, 0.3f});
    button_group->AnchorPoint({0.5f, 0.f});
    button_group->BGColor({0,0,0,0});
    button_group->SetBorderStyling(Colors::Cyans::Cyan, 1);

    auto *button_list = new JUI::VerticalListLayout(button_group);
    button_list->PaddingBottom(5_px);

    auto *new_world_btn = create_mainmenu_btn("New World", button_list);

    new_world_btn->OnReleaseEvent += [this](Vector2 pos, MouseButton btn, bool b)
    {
        RequestWorld.Invoke({.NewWorld = true});
    };

    auto *load_world_btn = create_mainmenu_btn("Load World", button_list);

    load_world_btn->OnReleaseEvent += [this](Vector2 pos, MouseButton btn, bool b)
    {
        RequestWorld.Invoke({.NewWorld = false});
    };


    //auto *sp_btn = create_mainmenu_btn("Singleplayer", button_list);

    //sp_btn->OnReleaseEvent += [this](Vector2 pos, MouseButton btn, bool b) {
    //    if (b)
    //        RequestWorld.Invoke({});
    //};

    auto *mp_btn = create_mainmenu_btn("Multiplayer", button_list);
    auto *credits_btn = create_mainmenu_btn("Credits", button_list);

    credits_btn->OnReleaseEvent += [this](Vector2 pos, MouseButton btn, bool b) {
        if (b)
            RequestShowCredits.Invoke();
    };

    // TODO: Replace with IconButton with steam logo
    auto *steam_community_btn = create_mainmenu_btn("Steam Community", button_list);
    auto *quit_btn = create_mainmenu_btn("Quit", button_list);


    quit_btn->OnReleaseEvent += [this](Vector2 pos, MouseButton btn, bool b)
    {
        if (b)
            RequestQuit.Invoke();
    };
}

void CaveGame::Client::MainMenu::PassWindowSize(const Vector2 &size) {
    Scene::PassWindowSize(size);
    scene->SetViewportSize(window_size);
}

void CaveGame::Client::MainMenu::PassMouseInput(unsigned int btn, bool down) {
    scene->ObserveMouseInput(static_cast<JUI::MouseButton>(btn), down);
}

void CaveGame::Client::MainMenu::PassMouseMovement(const Vector2 &pos) {
    goal = pos;
    scene->ObserveMouseMovement(pos);
}

