#pragma once

#include <J3ML/LinearAlgebra.hpp>
#include <vector>
#include "Color4.hpp"

namespace CaveGame::Core {
    class StatusEffect {
    };

    class Entity {
    public:
        int Health() const { return health; }

        int MaxHealth() const { return max_health; }

        virtual void Draw() = 0;

        virtual void Update(float elapsed) = 0;

    protected:
        int health;
        int max_health;
        int entity_instance_id;
        float age;
        Vector2 position;
        Vector2 bounding_box;
        std::vector<StatusEffect> active_effects;
        Color4 color;
    private:

    };

    class PhysicsEntity : public Entity {
    public:
        [[nodiscard]] Vector2 Velocity() const { return velocity; }

        [[nodiscard]] Vector2 EstimatedNextPosition() const { return next_position; }

    protected:
        Vector2 velocity;
        Vector2 next_position;
        float mass;
    private:


    };


    class Humanoid : public PhysicsEntity {
    public:
        virtual int BaseDefense() const;
        virtual int ArmorDefense() const
        {

        }
        virtual int DefenseModifier() const;
        int Defense() const
        {

        }
    protected:
    };

    class Projectile : public PhysicsEntity
    {

    };

    class Arrow : public Projectile {};
    class Fireball : public Projectile {};
    class Icecicle : public Projectile {};

    class Player : public Humanoid {
    public:
        void Jump();
        void Climb();
        void Descend();
        void Crouch();
        void WalkLeft();
        void WalkRight();
    protected:
    };



    class Zombie : public Humanoid {
    public:
    protected:
    };

    class ItemStack : PhysicsEntity {};

    class PhysicalParticle {};
}