#pragma once

#include <Core/Data.hpp>
#include <Core/Chunk.hpp>
#include <Core/Simplex.hpp>
#include <Core/Perlin.hpp>
#include <Core/Octave.hpp>

namespace CaveGame::Core
{

    // TODO: Terrain Generator Window Widget

    class Generator
    {
    public:
        static constexpr int PrecomputedWhiteNoiseResolution = 2048;
        static constexpr float BaseSurfaceLvl = 100.f;
        static constexpr float HeightMapHighPassAmplitude = 700.f;
        static constexpr float HeightMapHighPassScale = 700.f;
        static constexpr float HeightMapLowPassAmplitude = 85.f;
        static constexpr float HeightMapLowPassScale = 64.f;
        static constexpr float TopSoilDepth = 225.f;

        static constexpr float CaveInputScale = 800.f;
        static constexpr float CaveOutputScale = 1.25f;
        static constexpr float CaveErosionRange = 0.035f;
        static constexpr float CaveAdditiveInputScale = 80.f;
        static constexpr float CaveAdditiveOutputScale = 0.25f;

    public:
        Generator() = default;
        explicit Generator(int seed);

        TileID HeightMap(float depth, int wx, int wy);
        TileID ComputeTile(int wx, int wy);
        void FirstPass(Chunk& chunk);


        // TODO: Implement SecondPass, the catch is, it **may** need to load an arbitrary amount of adjacent chunks to correctly place structures.
        // TODO: Expert Mode: How do we keep it threaded separately while still accomplishing the above?

        uint PlaceTile();

        int ModInt(int x, int mod) const {

            x = Math::Abs(x);

            if (x == 0)
                return 0;

            return x % mod;
        }

        float GetPrecomputedWhiteNoise1D(int x) const;

        float GetPrecomputedWhiteNoise2D(int x, int y) const;

    protected:
        int seed = 0;
        SimplexNoise simplex;
        PerlinNoise perlin;

        RNG rng;
        float precomputed_white_noise_1D[PrecomputedWhiteNoiseResolution];
        float precomputed_white_noise_2D[PrecomputedWhiteNoiseResolution][PrecomputedWhiteNoiseResolution];
    private:
        float ComputeDepth(int wx, int wy);
    };
}