#include <set>
#include <Core/Registry.hpp>

#pragma once

namespace CaveGame::Core
{
    /// This enum defines combinatorial category flags to items to enable smart sorting.
    enum class ItemCategory: u8
    {
        ANY, TILE, ORE, TOOL, WEAPON, ARMOR, CLOTHING, FOOD, POTION, INGREDIENT, MEME, PLANT, METAL, ORGANIC, BUILDING_BLOCK,
        FLUID, SOIL, GAS, LIGHT_SOURCE, CRAFTING_STATION,
    };


    /// This enum defines a mutually-exclusive (one-at-a-time) 'Modifier' that items may randomly spawn with.
    /// It will affect item stats in a variety of ways, and some will even have special coding.
    enum class ItemModifiers : u8 {

        // These represent low-quality, crappy items, often with comical effects.
        FLIMSY, RUSTY, BROKEN, BENDY, RUBBERY, FRAGILE, FILTHY, DRAB, DULL, SCARRED, BURNED,

        // These primarily make an item more valuable.
        FABULOUS, GOLD_ENGRAVED, LOVELY, MAGNIFICENT,
        SPARKLING, PLAIN,
        // These primarily modify an item's visual and mechanical 'size'.
        TINY, LONG, GREATER,

        // These primarily modify an item's damage.
        SHARP, COMBATIVE,

        // These primarily modify an item's durability.
        HARDENED, FORGED, BRITTLE,  TOUGH,

        // These primarily affect an item's speed.
        LAZY, FRANTIC, LIGHT, HEAVY,

        // These primarily represent enchanted or magically imbued items.
        ELDRITCH, ARCANE, MYSTICAL, MYTHICAL, MAJESTIC,

        // These represent cursed or hexed items that have interesting drawbacks.
        ACCURSED, EVIL, MENACING, BLOODY, UGLY, FIERCE, NAUGHTY, DASTARDLY,

        // As-of-yet uncategorized modifiers.
        ANCIENT, VICTORIOUS, CONDEMNED,  CLEAN, LUCKY, POWERFUL, SUPER,
    };

    class Item
    {
    public:

    protected:
    private:
    };

    class EmptyBottle : public Item
    {
        REGISTER("EmptyBottle", Item);
    };

    class ItemFilter
    {

        std::set<ItemCategory> AllowedCategories;
        std::set<ItemCategory> BannedCategories;

        std::set<std::string> AllowedItems;
        std::set<std::string> BannedItems;
    };


}

