#include <Core/Chunk.hpp>
#include "Core/Perlin.hpp"
#include "Core/Simplex.hpp"
#include "Core/Data.hpp"
#include <iostream>

namespace CaveGame::Core
{
    void Chunk::SetTile(int x, int y, TileID t) {
        if (!(0 <= x && x < ChunkSize && 0 <= y && y < ChunkSize))
            assert("Out of bounds!");

        if (tiles[x][y] != t)
        {
            tiles[x][y] = t;
            touched = true;
        }
    }

    TileID Chunk::GetTile(int x, int y) const {
        if (!(0 <= x && x < ChunkSize || 0 <= y && y < ChunkSize))
            throw std::runtime_error("Out of bounds!");
        return tiles[x][y];
    }

    Chunk::Chunk(const Vector2 &cell_coords) : cell(cell_coords)
    { }

    Vector2 Chunk::GetChunkRealCoordinates() const {
        return GetChunkCell()*Core::Chunk::ChunkSize;
    }

    Vector2 Chunk::GetChunkCell() const { return cell;}

}
