#include <Core/Interfaces.hpp>

namespace CaveGame::Core
{
    bool ITileMap::IsNonSolidTile(int x, int y) const
    {
        return GetTile(x, y) == TileID::AIR;
    }

    bool ITileMap::IsSolidTile(int x, int y) const
    {
        return !IsNonSolidTile(x, y);
    }

    bool ITileMap::HasAdjacentAirBlock(int x, int y) const
    {
        if (IsNonSolidTile(x+1, y) || IsNonSolidTile(x-1, y) || IsNonSolidTile(x, y+1) || IsNonSolidTile(x, y-1))
            return true;
        return false;
    }

    bool ITileMap::HasAdjacentOrDiagonalAirBlock(int x, int y) const
    {
        if (IsNonSolidTile(x+1, y) || IsNonSolidTile(x-1, y) || IsNonSolidTile(x, y+1) || IsNonSolidTile(x, y-1) ||
            IsNonSolidTile(x+1, y-1) || IsNonSolidTile(x-1, y+1) || IsNonSolidTile(x+1, y+1) || IsNonSolidTile(x-1, y-1))
            return true;
        return false;
    }

    bool ITileMap::GrassSpreadable(int x, int y) const
    {
        if (GetTile(x, y) == TileID::DIRT)
        {
            if (HasAdjacentOrDiagonalAirBlock(x, y))
                return true;
        }
        return false;

    }

    bool ITileMap::GrassShouldSuffocate(int x, int y) const
    {
        //if (IsSolidTile(x+1, y) && IsSolidTile(x-1, y) && IsSolidTile(x, y+1) && IsSolidTile(x, y-1) &&
        //IsSolidTile(x+1, y+1) && IsSolidTile(x-1, y+1) && IsSolidTile(x+1, y-1) && IsSolidTile(x-1, y-1))
        if (!HasAdjacentOrDiagonalAirBlock(x, y))
            return true;
        return false;
    }

    void ITileMap::GrassRandomTicc(int wx, int wy)
    {
        if (GrassShouldSuffocate(wx, wy))
        {
            SetTile(wx, wy, TileID::DIRT);
            return;
        }

        // Check adjacent blocks
        if (GrassSpreadable(wx, wy+1))
        {
            SetTile(wx, wy+1, TileID::GRASS);
            return;
        }


        if (GrassSpreadable(wx, wy-1))
        {
            SetTile(wx, wy-1, TileID::GRASS);
            return;
        }


        if (GrassSpreadable(wx+1, wy))
        {
            SetTile(wx+1, wy, TileID::GRASS);
            return;
        }


        if (GrassSpreadable(wx-1, wy))
        {
            SetTile(wx-1, wy, TileID::GRASS);
            return;
        }


        // Check diagonal blocks.
        if (GrassSpreadable(wx+1, wy-1))
        {
            SetTile(wx+1, wy-1, TileID::GRASS);
            return;
        }


        if (GrassSpreadable(wx-1, wy-1))
        {
            SetTile(wx-1, wy-1, TileID::GRASS);
            return;
        }


        if (GrassSpreadable(wx+1, wy+1))
        {
            SetTile(wx+1, wy+1, TileID::GRASS);
            return;
        }


        if (GrassSpreadable(wx-1, wy+1))
        {
            SetTile(wx-1, wy+1, TileID::GRASS);
            return;
        }

    }
}
